(function() {
	'use strict';

	angular
		.module('psa.app.controller', [])
		.controller('AppController', AppController);

	AppController.$inject = ['$log', '$cordovaBarcodeScanner', '$ionicPopup', 'User', 'Talk', 'auth', '$state'];

	/* @ngInject */
	function AppController($log, $cordovaBarcodeScanner, $ionicPopup, User, Talk, auth, $state) {
		var vm = this;

		vm.scanQRCode = scanQRCode;
		vm.promptQRCode = promptQRCode;
		vm.getPictureURL = getPictureURL;

		activate();

		////////////////

		function addTalk(talkId) {
			vm.user.talks = vm.user.talks || {};
			vm.user.talks.asAttendant = vm.user.talks.asAttendant || {};
			if (vm.user.talks.asAttendant[talkId] ) {
				$ionicPopup.alert({
					title: 'Palestra já foi adicionada',
					template: 'Esta palestra já consta em sua lista de palestras.',
					okType: 'button-energized'
				}).then(function() {
					goTalk(talkId)
				})
				return;
			}

			var talk = Talk(talkId);
			talk.$loaded().then(function () {
				talk.attendants = talk.attendants || {};
				talk.attendants[vm.user.$id] = true;
				return talk.$save();
			}).then(function() {
				vm.user.talks.asAttendant[talkId] = true;
				// add to user interests and remove duplicates (filter)
				vm.user.interests = (vm.user.interests || []).concat(talk.categories || []).filter(function(elem, pos, arr) {
					return arr.indexOf(elem) == pos;
			  });
				// add interests as OneSignal tags
				if (window.plugins && window.plugins.OneSignal) {
					var tagmap = vm.user.interests.reduce(function(prev, tag) {
						prev[tag] = 1;
						return prev;
					}, {});
					window.plugins.OneSignal.sendTags(tagmap);
				}
				// save user
				return vm.user.$save();
			}).then(function () {
				$ionicPopup.alert({
					title: 'Palestra Adicionada',
					template: 'A palestra foi incluída em sua lista de presença com sucesso.',
					okType: 'button-energized'
				}).then(function() {
					goTalk(talkId)
				})
			}).catch(function (err) {
				console.log(err);
				$ionicPopup.alert({
					title: 'Desculpe',
					template: 'Houve um erro não identificado e não foi possível adicionar esta palestra. Por favor, tente novamente.',
					okType: 'button-energized'
				});
			});
		}

		function goTalk(talkId) {
			$state.go("app.menu.talk", { id: talkId })
		}

		function talkNotExists() {
			$ionicPopup.alert({
				title: 'Desculpe',
				template: 'Não foi encontrada nenhuma palestra a partir deste código.',
				okType: 'button-energized'
			});
		}

		function talkInvalidCode() {
			$ionicPopup.alert({
				title: 'Desculpe',
				template: 'Código inválido para esta palestra.',
				okType: 'button-energized'
			});
		}

		function successQRCode(data) {
			if (data && data.text) {
				var talkId = data.text;
				console.log("QRCODE", data.text);
				var ref = firebase.database().ref().child('talks').child(talkId);
				ref.once('value', function (snap) {
					if (snap.exists()) {
						addTalk(talkId);
					} else {
						talkNotExists();
					}
				});
			} else {
				// user cancelled the QR scanning
			}
		}

		function promptQRCode(talkId) {
			$ionicPopup.prompt({
				cssClass: 'popup-talk-code',
				title: 'Código da Palestra',
				template: 'Digite o código da Palestra',
				//inputType: 'password',
				inputPlaceholder: 'Código da Palestra',
				cancelText: 'Cancelar',
				okText: 'Check-in',
				okType: 'button-energized'
			 }).then(function(res) {
			 	if (res) {
			 		var code = latinize(String(res).trim()).toUpperCase();
				  checkTalkCode(talkId, code);
				}
			 });			
		}

		function checkTalkCode(talkId, code) {
			var talk = Talk(talkId);
			talk.$loaded().then(function () {
				if (talk.code === code) {
					addTalk(talkId);
				} else {
					talkInvalidCode();
				}
			}).catch(function(ex) {
				talkNotExists();
			})
		}

		function scanQRCode() {
			$cordovaBarcodeScanner.scan({
				showFlipCameraButton: true,
				prompt: 'Alinhe o QR Code ao retângulo na câmera.',
				formats: 'QR_CODE,PDF_417'
			}).then(successQRCode).catch(function (err) {
				var notAvailable = (err === "cordova_not_available");
				if (notAvailable) {
					promptQRCode();
					return;
				}
				$log.error(err);
				$ionicPopup.alert({
					title: 'Desculpe',
					template: 'Não foi possível realizar a leitura do QR Code. Por favor, tente novamente',
					okType: 'button-energized'
				});
			});
		}

		function getUserInfo() {
			vm.uid = auth.uid;
			vm.user = User(auth.uid);
      // set onesignal user tag
      if (window.plugins && window.plugins.OneSignal) {
      	window.plugins.OneSignal.sendTag("_userId", vm.uid);
      }
		}

		function getPictureURL(folder, name) {
			var storage = firebase.storage();
			var pathReference = storage.ref(folder + '/' + name + '.jpg');
			console.log(pathReference.fullPath);
			return pathReference;
		}

		function activate() {
			getUserInfo();
		}
	}
})();
