(function() {
	'use strict';

	angular
		.module('psa.auth.login.controller', [])
		.controller('LoginController', LoginController);

	LoginController.$inject = ['Auth', '$scope', '$ionicPopup', '$ionicLoading'];

	/* @ngInject */
	function LoginController(Auth, $scope, $ionicPopup, $ionicLoading) {
		var vm = this;

		vm.login = login;
		vm.forgot = forgot;

		activate();

		////////////////
		function validateEmail(email) {
			var re = /^(([^<>()\[\]\\.,;:\s@"]+(\.[^<>()\[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;
			return re.test(email);
		}

		function login() {
			if (!validateEmail(vm.user.email)) {
				var subTitle = 'E-mail inválido';
				var template = 'Por favor, forneça um e-mail válido';
				$ionicPopup.alert({
					title: 'Desculpe',
					subTitle: subTitle,
					template: template,
					okType: 'button-energized'
				});
				return;				
			}

			$ionicLoading.show({
				template: 'Autenticando...'
			});
			Auth.$signInWithEmailAndPassword(vm.user.email, vm.user.password).then(function (result) {
				// set onesignal user tag
				if (window.plugins && window.plugins.OneSignal) {
					window.plugins.OneSignal.sendTag("_userId", result.uid);
				} else {
					console.log("ONESIGNAL NOT FOUND");
				}

				vm.user = {};
			}).catch(function (err) {
				var subTitle;
				var template;
				console.log(err.code);
				switch(err.code) {
					case 'auth/invalid-email':
					subTitle = 'E-mail inválido';
					template = 'Por favor, forneça um e-mail válido';
					break;
					case 'auth/user-disabled':
					subTitle = 'Este usuário está desativado';
					template = 'Por favor, contate um administrador para reativar o seu usuário.';
					break;
					case 'auth/user-not-found':
					subTitle = 'Usuário não encontrado';
					template = 'O e-mail fornecido não está cadastrado em nosso sistema.';
					break;
					case 'auth/wrong-password':
					subTitle = 'Senha não confere';
					template = 'A senha fornecida está incorreta.';
					break;
					case 'auth/operation-not-allowed':
					subTitle = 'Login não confere';
					template = 'Email ou senha inválidos';
					break;
					case 'auth/network-request-failed':
					subTitle = 'Erro de rede';
					template = 'Não foi possível conectar ao sistema. Por favor, verifique sua conexão.';
					break;
					default:
					subTitle = 'Erro';
					template = 'Não foi possível realizar o login neste momento. Tente mais tarde.';
					break;
				}
				$ionicPopup.alert({
					title: 'Desculpe',
					subTitle: subTitle,
					template: template,
					okType: 'button-energized'
				});
			}).finally(function () {
				$ionicLoading.hide();
			});
		}

		function forgot() {
			var forgotPopup = $ionicPopup.show({
				template: '<input type="email" ng-model="login.forgotEmail">',
				title: 'Esqueceu a senha?',
				subTitle: 'Forneça o seu e-mail:',
				scope: $scope,
				buttons: [
					{ text: 'Cancelar' },
					{
						text: 'Enviar',
						type: 'button-energized',
						onTap: function (e) {
							if (!vm.forgotEmail) {
								e.preventDefault();
							} else {
								return vm.forgotEmail;
							}
						}
					}
				]
			});

			forgotPopup.then(function (res) {
				if (res) {
					Auth.$sendPasswordResetEmail(res).then(function () {
						$ionicPopup.alert({
							title: 'Sua senha foi resetada',
							template: 'Por favor, confira o seu e-mail com a nova senha',
							okType: 'button-energized'
						});
					}).catch(function (err) {
						var subTitle;
						var template;
						console.log(err);
						switch (err.code) {
							case 'auth/invalid-email':
								subTitle = 'E-mail inválido';
								template = 'Por favor, forneça um e-mail válido';
								break;
							case 'auth/user-not-found':
								subTitle = 'Usuário não encontrado';
								template = 'Este e-mail não consta nos cadastros deste aplicativo.';
								break;
						}
						$ionicPopup.alert({
							title: 'Desculpe',
							subTitle: subTitle,
							template: template,
							okType: 'button-energized'
						});
					});
				}
			});
		}

		function activate() {
			vm.user = {};
			console.log(Auth);
		}
	}
})();
