(function() {
	'use strict';

	angular
		.module('psa.auth.signup.controller', [])
		.controller('SignupController', SignupController);

	SignupController.$inject = ['$log', '$ionicLoading', '$timeout', 'UploadImage', 'Auth', 'User', '$ionicPopup', '$ionicActionSheet', '$cordovaCamera', '$q'];

	/* @ngInject */
	function SignupController($log, $ionicLoading, $timeout, UploadImage, Auth, User, $ionicPopup, $ionicActionSheet, $cordovaCamera, $q) {
		var vm = this;

		vm.signup = signup;
		vm.selectPicture = selectPicture;
		
		activate();

		////////////////

		function validate() {
			var errors = [];
			if (!vm.picture) {
				errors.push('<strong>Foto</strong> é obrigatório');
			}
			if (!vm.user.name) {
				errors.push('<strong>Nome</strong> é obrigatório');
			}
			if (!vm.user.email) {
				errors.push('<strong>E-mail</strong> é obrigatório');				
			}
			if (!vm.user.phone) {
				errors.push('<strong>Telefone</strong> é obrigatório');				
			}
			if (!vm.user.password || vm.user.password.length < 6) {
				errors.push('<strong>Senha</strong> deve conter no mínimo 6 caracteres');
			}
			if (vm.user.password !== vm.confirmPassword) {
				errors.push('<strong>Senha</strong> e <strong>Confirmação de Senha</strong> não conferem.');
			}
			if (!errors.length) {
				return true;
			}
			var template = '';
			for (var e in errors) {
				var error = errors[e];
				template += error + '<br>';
			}
			$ionicPopup.alert({
				title: 'Desculpe',
				subTitle: 'Houve um ou mais erros em seu cadastro',
				template: template,
				okType: 'button-energized'

			});
			return false;
		}

		function signup() {
			if (validate()) {
				$ionicLoading.show({
					template: 'Cadastrando usuário...'
				});
				var user = angular.copy(vm.user);
				var newUser;
				var uid;
				Auth.$createUserWithEmailAndPassword(vm.user.email, vm.user.password).then(function (firebaseUser) {
					delete user.password;
					uid = firebaseUser.uid;
					newUser = User(uid);
					for (var prop in user) {
						newUser[prop] = user[prop];
					}
					return newUser.$save();
				}).then(function() {
					if (vm.picture) {
						return UploadImage.uploadImage(vm.pictureUrl, 'pictures', uid, function () {
						}).then(function(pictureUrl) {
							newUser.pictureUrl = pictureUrl;
							return newUser.$save();
						});
					}
					return $q.resolve(true);
				}).then(function () {
					return Auth.$signInWithEmailAndPassword(vm.user.email, vm.user.password);
				}).then(function (result) {
					// set onesignal user tag
					window.plugins.OneSignal.sendTag("_userId", result.uid);
					vm.user = {};
					delete vm.confirmPassword;
				}).catch(function (err) {
					var subTitle;
					var template;
					console.log(JSON.stringify(err));
					switch (err.code) {
						case 'auth/email-already-in-use':
							subTitle = 'E-mail já existe';
							template = 'Use outro e-mail ou volte à pagina de login e use este e-mail.';
							break;
						case 'auth/invalid-email':
							subTitle = 'E-mail inválido';
							template = 'Por favor, forneça um e-mail válido para prosseguir com o cadastro;';
							break;
						case 'auth/operation-not-allowed':
							subTitle = 'Operação não permitida';
							template = 'Houve um erro não esperado. Por favor, contate os administradores.';
							break;
						case 'auth/weak-password':
							subTitle = 'Senha não é forte suficiente';
							template = 'Por favor, use uma senha mais forte.';
							break;
						default:
							subTitle = 'Erro não esperado';
							template = 'Ao persistir o erro, por favor, contate os administradores do aplicativo';
							break;
					}
					$ionicPopup.alert({
						title: 'Desculpe',
						subTitle: subTitle,
						template: template, 
						okType: 'button-energized'
					});
				}).finally(function () {
					$ionicLoading.hide();
				});
			}
		}

		function getPicture(type) {
			$cordovaCamera.getPicture({
				destinationType: $cordovaCamera.DestinationType.FILE_URI,
				encodingType: $cordovaCamera.EncodingType.JPEG,
				sourceType: type,
				allowEdit: true,
				targetWidth: 250,
				targetHeight: 250
			}).then(function (res) {
				window.resolveLocalFileSystemURL(res, function (fileEntry) {
					$timeout(function () {
						var nativeURL = fileEntry.nativeURL;
						var path = '/' + nativeURL.replace(cordova.file.applicationStorageDirectory, '');
						vm.picture = path;
						vm.pictureUrl = nativeURL;
						console.log("FILE ENTRY", JSON.stringify(fileEntry));
						console.log("PICTURE", vm.picture);
						console.log("PICTURE URL", vm.pictureUrl);
					});
				});
			}).catch(function (error) {
				$log.error(error);
			});
		}

		function selectPicture() {
			$ionicActionSheet.show({
				buttons: [
					{ text: '<i class="icon ion-camera dark"></i> Selecionar da câmera' },
					{ text: '<i class="icon ion-image energized"></i> Selecionar do álbum' }
				],
				titleText: 'Sua foto',
				cancelText: 'Cancelar',
				buttonClicked: function (index) {
					var type;
					switch (index) {
						case 0:
							type = $cordovaCamera.PictureSourceType.CAMERA;
							break;
						case 1:
							type = $cordovaCamera.PictureSourceType.PHOTOLIBRARY;
							break;
					}
					getPicture(type);
					return true;
				}
			});
		}

		function activate() {
			vm.user = {};
		}
	}
})();
