(function() {
	'use strict';

	angular
		.module('psa.menu.controller', [])
		.controller('MenuController', MenuController);

	MenuController.$inject = ['Auth', 'User', '$state', 'auth', '$cordovaSocialSharing', '$ionicHistory', '$ionicSideMenuDelegate', '$scope', 'TalkService', 'EmailService', '$ionicPopup', '$ionicLoading'];

	/* @ngInject */
	function MenuController(Auth, User, $state, auth, $cordovaSocialSharing, $ionicHistory, $ionicSideMenuDelegate, $scope, TalkService, EmailService, $ionicPopup, $ionicLoading) {
		var vm = this;

		vm.share = share;
		vm.logout = logout;
		vm.goProfile = goProfile;
		vm.close = close;
		vm.contactUs = contactUs;
		vm.home = home;
		vm.subscribed = subscribed;
		vm.lecturing = lecturing;
		vm.isSpeaker = false;

		activate();

		$scope.$watch('menu.speakerMode', function(val) {
			if (val !== undefined) {
				localStorage.setItem("speakerMode", JSON.stringify(val));
				if (val) {
					lecturing();
				} else {
					home();
				}
			}
		});

		////////////////

		function share() {
			$cordovaSocialSharing.share('Veja o aplicativo da Profissionais S/A. Perfeito para palestras e palestrantes!', 'Soluções em palestras e palestrantes', null, 'http://www.profissionaissa.com/aplicativo');
		} 

		function logout() {
			$scope.$on('$ionicView.afterLeave', function () {
				$ionicHistory.clearCache();
				$ionicHistory.clearHistory();
			});
			Auth.$signOut();
		}

		function getUserInfo() {
			var uid = Auth.$getAuth().uid;
			vm.user = User(uid);
			return vm.user.$loaded();
		}
		function close() {
			$ionicSideMenuDelegate.toggleLeft();
		}

		function home() {
			close();
			$ionicHistory.nextViewOptions({
			  disableAnimate: true,
			  disableBack: true
			});
			$state.go("app.menu.talks").then(function() {
				try {
					$ionicHistory.removeBackView()
				} catch (ex) {}
			})
		}

		function subscribed() {
			close();
			$ionicHistory.nextViewOptions({
			  disableAnimate: true,
			  disableBack: true
			});
			$state.go("app.menu.subscribed");
		}

		function lecturing() {
			close();
			$ionicHistory.nextViewOptions({
			  disableAnimate: true,
			  disableBack: true
			});
			$state.go("app.menu.lecturing");
		}

		function goProfile() {
			close();
			$state.go('app.menu.userEdit');
		}

		function getLastTalk() {
			TalkService.onLastTalkChanged(function (talk) {
				vm.talk = talk;
			});
		}

		function setupSpeakerMode(isSpeaker) {
			if (!isSpeaker) {
				localStorage.removeItem("speakerMode");
				return;
			}
			var speakerMode = localStorage.getItem("speakerMode");
			if (speakerMode === null) {
				vm.speakerMode = isSpeaker;
			} else {
				vm.speakerMode = isSpeaker && JSON.parse(speakerMode);
			}
		}

		function lifeCycleEvents() {
			$scope.$on('$ionicView.beforeEnter', function (event, data) {
				getUserInfo().then(function(user) {
					vm.isSpeaker = !!user.speaker;
					setupSpeakerMode(user.speaker);
				})
				getLastTalk();
			});
		}

		/*
		function contactUs() {
		  var email = {
		    to: 'contato@profissionaissa.com',
		    subject: 'Sobre o aplicativo Profissionais S/A',
		    body: 'Escreva aqui seu feedback para nossa equipe!',
		    isHtml: false
		  };

			$cordovaEmailComposer.isAvailable().then(function() {
			  return $cordovaEmailComposer.open(email);
			}, function () {
			  // not available
			});
		}
		*/

		function escapeHTML(html) {
	    var pre = document.createElement('pre');
	    var text = document.createTextNode(html);
	    pre.appendChild(text);
	    return pre.innerHTML;
		}

	  function promptSend(title, subtitle) {
	    $scope.promptSend = {};

	    // An elaborate, custom popup
	    var myPopup = $ionicPopup.show({
	      template: '<textarea rows="15" ng-model="promptSend.response"></textarea>',
	      title: title,
	      subTitle: subtitle,
	      scope: $scope,
	      buttons: [
	        { 
	          text: 'Cancelar',
	          type: 'button-default'
	        },
	        { 
	          text: 'Enviar',
	          type: 'button-positive',
	          onTap: function(e) {
	            return $scope.promptSend.response;
	          }        
	        }
	      ]
	    });  
	    return myPopup;
	  }

	  function promptSendEmail(title, subtitle, email) {
	    promptSend(title, subtitle).then(function(res) {
	      if (res && String(res).trim().length > 0) {
	        $ionicLoading.show();
	      	EmailService.send([{
	      			email: email.to,
	      			name: "Contato Profissionais S/A"
	      		}], email.templateId, {
	      		"%USER_NAME%": vm.user.name,
	      		"%USER_EMAIL%": vm.user.email,
	      		"%MESSAGE%": escapeHTML(res).replace(/\n/g, "<br>\n")
	      	}, {
	      		email: vm.user.email,
	      		name: vm.user.name
	      	}).then(function() {
	          promptAlert("Mensagem enviada com sucesso!");
	        }).catch(function() {
	          promptAlert("Não foi possível enviar a mensagem neste momento");
	        }).finally(function() {
	          $ionicLoading.hide();
	        });
	      }
	    });
	  }

	  function promptAlert(msg) {
	    $ionicPopup.alert({
	      template: msg
	    });
	  }

	  function contactUs() {
		  var email = {
		    to: 'contato@profissionaissa.com',
		    user: vm.user,
		    templateId: 'dba72999-fd0d-4f22-a3fd-804c46ca3adf'
		  };
	    promptSendEmail("Deixe seu comentário", "Mensagem:", email);
	  }

		function activate() {
			lifeCycleEvents();
		}
	}
})();
