(function() {
	'use strict';

	angular
		.module('psa.questions.controller', [])
		.controller('QuestionsController', QuestionsController);

	QuestionsController.$inject = ['$scope', '$ionicScrollDelegate', 'Questions', 'Question', '$ionicPopup', 'auth', 'Talk', '$ionicLoading', '$stateParams', 'User', 'NotificationService'];

	/* @ngInject */
	function QuestionsController($scope, $ionicScrollDelegate, Questions, Question, $ionicPopup, auth, Talk, $ionicLoading, $stateParams, User, NotificationService) {
		var vm = this;

		var FEATURE_BLOCK_CHAT_WHEN_EXPIRED = false;

		vm.answer = answer;
		vm.answerInPerson = answerInPerson;
		vm.ask = ask;
		vm.expired = false;

		activate();

		////////////////

		function answerInPerson(qId) {
			var talkId = $stateParams.id;
			var question = Question(talkId, qId);
			question.$loaded().then(function () {
				question.answer = "Respondido presencialmente";
				question.answerInPerson = true;
				question.$save().then(function () {
					delete vm.answerString;
					sendPushAnswerInPerson(question);
				});
			});			
		}

		function answer(qId) {
			var popup = $ionicPopup.show({
				template: '<textarea placeholder="Preencha sua resposta" ng-model="questions.answerString"></ŧextarea>',
				title: 'Responder',
				subTitle: 'Preencha a resposta na caixa abaixo',
				scope: $scope,
				buttons: [
					{ text: 'Cancelar' },
					{ 
						text: 'Responder', 
						type: 'button-energized',
						onTap: function (e) {
							if (!vm.answerString) {
								e.preventDefault();
							} else {
								return vm.answerString;
							}
						}
					}
				]
			});
			popup.then(function (ans) {
				var talkId = $stateParams.id;
				var question = Question(talkId, qId);
				question.$loaded().then(function () {
					question.answer = ans;
					question.$save().then(function () {
						delete vm.answerString;
						sendPushAnswer(question);
					});
				});
			});
		}

		function sendPushToSpeaker(question) {
			var speakerId = vm.speaker.$id;
			var msg = "Pergunta: " + question.question;
			NotificationService.sendToUser(speakerId, msg).success(function(data, status) {
      	console.log("PUSH", JSON.stringify(data));
      }).error(function(err) {
        console.log("PUSH ERR", JSON.stringify(err));
      });
		}

		function sendPushAnswer(question) {
			var userId = question.user;
			var msg = "Resposta: " + question.answer;
			NotificationService.sendToUser(userId, msg).success(function(data, status) {
      	console.log("PUSH", JSON.stringify(data));
      }).error(function(err) {
        console.log("PUSH ERR", JSON.stringify(err));
      });
		}

		function sendPushAnswerInPerson(question) {
			var userId = question.user;
			var msg = "Sua pergunta foi respondida presencialmente";
			NotificationService.sendToUser(userId, msg).success(function(data, status) {
      	console.log("PUSH", JSON.stringify(data));
      }).error(function(err) {
        console.log("PUSH ERR", JSON.stringify(err));
      });
		}

		function ask() {
			var talkId = $stateParams.id;
			var question = new Question(talkId);
			question.question = vm.question;
			question.user = auth.uid;
			question.timestamp = (new Date()).getTime();
			question.$save().then(function () {
				delete vm.question;
				$ionicScrollDelegate.scrollBottom();
				sendPushToSpeaker(question);
			});
		}

		function getQuestions() {
			var talkId = $stateParams.id;
			var expired = JSON.parse($stateParams.expired || "false");
			if (FEATURE_BLOCK_CHAT_WHEN_EXPIRED) {
				vm.expired = expired;
			}

			$ionicLoading.show({
				template: 'Carregando questões...'
			});
			vm.list = new Questions(talkId);
			vm.list.$loaded().finally(function () {
				$ionicLoading.hide();
				$ionicScrollDelegate.scrollBottom();
			});

		}

		function getSpeaker() {
			vm.speaker = new User(vm.talk.speaker);
			vm.speaker.$loaded().then(function () {
				$ionicScrollDelegate.scrollBottom();
			});
		}

		function getTalk() {
			var talkId = $stateParams.id;
			vm.talk = Talk(talkId);
			vm.talk.$loaded().then(function () {
				getSpeaker();
				if (vm.talk.speaker === auth.uid) {
					vm.amISpeaker = true;
					$ionicScrollDelegate.scrollBottom();
				}
			});
		}

		function activate() {
			getTalk();
			getQuestions();
		}
	}
})();
