(function() {
  'use strict';

  angular
      .module('psa.shared.facebookapi.service', [])
      .service('FacebookAPIService', ['$http', '$q', '$timeout', FacebookAPIService]);

  function FacebookAPIService($http, $q, $timeout) {

    function encodeURIComponents(obj) {
      var str = "";
      for (var key in obj) {
          if (str != "") {
              str += "&";
          }
          str += key + "=" + encodeURIComponent(obj[key]);
      }
      return str;
    }

    this.setAccessToken = function(accessToken) {
      localStorage.setItem("fb_access_token", accessToken);
    }

    this.getAccessToken = function() {
      return localStorage.getItem("fb_access_token");
    }

    this.removeAccessToken = function() {
      if (this.getAccessToken()) {
        localStorage.removeItem("fb_access_token");
      }
    }

    this.get = function(url, params) {
      var baseUrl = "https://graph.facebook.com";
      //console.log("FB GET", baseUrl + "/v2.8" + url);
      var uriParams = encodeURIComponents(params || {});
      return $http({
          method: "GET",
          url: baseUrl + "/v2.8" + url + "?access_token=" + this.getAccessToken() + "&" + uriParams,
          headers: {
              "Content-Type": "application/json"
          }
      });
    }

    this.nextPage = function(prevResult) {
      //console.log("FB GET", prevResult.data.paging.next);
      return $http({
          method: "GET",
          url: prevResult.data.paging.next,
          headers: {
              "Content-Type": "application/json"
          }
      });
    }

    this.getAllPages = function(url) {
      var pages = [];

      function checkNextPage(result) {
        if (result.data.paging && result.data.paging.next) {
          return $timeout(function() {
            return self.nextPage(result).then(function(result) {
              pages.push(result);
              return checkNextPage(result) || $q.resolve(pages);
            }, function(err) {
              return $q.reject(err);
            });
          }, 800);
        }
      }

      var self = this;
      return this.get(url).then(function(result) {
        pages.push(result);
        return checkNextPage(result) || $q.resolve(pages);
      }, function(err) {
        $q.reject(err);
      });
    }
  }

}());
