(function() {
	'use strict';

	angular
		.module('psa.shared.uploadImage.service', [])
		.factory('UploadImage', UploadImage);

	UploadImage.$inject = ['$q', '$cordovaFile', '$timeout'];

	/* @ngInject */
	function UploadImage($q, $cordovaFile, $timeout) {
		var service = {
			uploadImage: uploadImage
		};
		return service;

		////////////////
		function getFileBlob(url, cb) {
			var path = url.substring(0, url.lastIndexOf('/') + 1);
			var filename = url.substring(url.lastIndexOf('/') + 1, url.length);
			$cordovaFile.readAsArrayBuffer(path, filename)
				.then(function (success) {
					var blob = new Blob([success], { type: 'image/jpeg' });
					cb(blob);
				}, function (error) {
					console.error(error);
					cb(null);
				});
		}

		function blobToFile(blob) {
			blob.lastModifiedDate = new Date();
			blob.name = name;
			return blob;
		}

		function getFileObject(filePathOrUrl, filename, cb) {
			console.log('Getting file object');
			getFileBlob(filePathOrUrl, function (blob) {
				if (blob) {
					cb(blobToFile(blob, filename + '.jpg'));				
				} else {
					cb(null);
				}
			});
		}

		function uploadImage(file, folder, filename, progressFn) {
			var defer = $q.defer();
			console.log('Uploading');
			getFileObject(file, filename, function (fileObject) {
				if (!fileObject) {
					return defer.reject();
				}
				var metadata = { contentType: fileObject.type };
				var storageRef = firebase.storage().ref();
				var uploadFile = storageRef.child(folder + '/' + filename).put(fileObject, metadata);
				uploadFile.on(firebase.storage.TaskEvent.STATE_CHANGED,
					function (snapshot) {
						var progress = (snapshot.bytesTransferred / snapshot.totalBytes) * 100;
						$timeout(function () {
							progressFn(progress);
							console.log(progress);
						});
					},

					function (err) {
						defer.reject(err);
					},

					function () {
						var downloadURL = uploadFile.snapshot.downloadURL;
						defer.resolve(downloadURL);
					}
				);
			});
			return defer.promise;
		}
	}
})();
