(function() {
	'use strict';

	angular
		.module('psa.talk.controller', [])
		.controller('TalkController', TalkController);

	TalkController.$inject = ['Talk', '$scope', 'auth', '$timeout', '$stateParams', '$state', '$ionicPopup', '$ionicLoading', '$cordovaSocialSharing', '$cordovaActionSheet', 'User', 'Attendants', 'Questions', 'TalkService', 'EmailService', 'NotificationService', '$cordovaInAppBrowser'];

	/* @ngInject */
	function TalkController(Talk, $scope, auth, $timeout, $stateParams, $state, $ionicPopup, $ionicLoading, $cordovaSocialSharing, $cordovaActionSheet, User, Attendants, Questions, TalkService, EmailService, NotificationService, $cordovaInAppBrowser) {
		var vm = this;

		vm.getSlides = getSlides;
		vm.getCertificate = getCertificate;
		vm.setLocal = setLocal;
		vm.rate = rate;
		vm.viewRatings = viewRatings;
		vm.user = User(auth.uid);
		vm.share = shareTalk;
		vm.test = test;
		vm.setScrolled = setScrolled;
		vm.getScrolled = getScrolled;
		vm.isExpired = isExpired;

		activate();

		////////////////

		function test() {
			var msg = "Testando notificações";
			console.log("ENVIANDO", vm.user.$id);
			NotificationService.sendToUser(vm.user.$id, msg).success(function(data, status) {
      	console.log("PUSH", JSON.stringify(data));
      }).error(function(err) {
        console.log("PUSH ERR", JSON.stringify(err));
      });			
		}

		function getScrolled() {
			return vm.scrolled;
		}

		function setScrolled(value) {
			$timeout(function() {
				vm.scrolled = value;
			})
		}

		function getSlides() {
			if (isExpired()) {
				return showExpirationAlert();
			}			
			var options = {
			  //title: 'What do you want with this image?',
			  buttonLabels: ['Ver slides', 'Compartilhar slides'],
			  addCancelButtonWithLabel: 'Cancelar',
			  androidEnableCancelButton : true,
			  winphoneEnableCancelButton : true,
			  //addDestructiveButtonWithLabel : 'Delete it'
			};
  		$cordovaActionSheet.show(options).then(function(btnIndex) {
  			if (btnIndex === 1) {
  				viewSlides();
  			} else if (btnIndex === 2) {
					shareSlides();
  			}
  		}, false);
		}

		function shareSlides() {
			var message = "Slides da palestra \"" + vm.obj.title + "\" de " + vm.speaker.name + ": " + vm.obj.description + " #ProfissionaisSA";
			var subject = "Slides da palestra \"" + vm.obj.title + "\"";
			var file = null;
			var link = vm.obj.slides.url;
			$cordovaSocialSharing
			.share(message, subject, file, link) // Share via native share sheet
			.then(function(result) {
				// Success!
			}, function(err) {
				// An error occured. Show a message to the user
			});			
		}

		function viewSlides() {
			if (!vm.obj.slides.url) {
				$ionicPopup.alert({
					title: 'Desculpe',
					template: 'Os slides dessa palestra ainda não estão disponíveis'
				});
			} else {
				console.log(vm.obj.slides.url);
				var options = {
					location: 'no',
					clearcache: 'no',
					toolbar: 'yes'
				};
				// var html = '<html><body class="margin:0;padding:0"><iframe src="' + vm.obj.slides.url + '" style="width:100%; height:100%; border:none; margin:0; padding:0;"></iframe></body></html>';
				// cordova.InAppBrowser.open('data:text/html,' + encodeURI(html), '_blank', options);
				//cordova.InAppBrowser.open(vm.obj.slides.url, '_blank', options);
				var isIOS = ionic.Platform.isIOS();
				var url = (isIOS) ? vm.obj.slides.url :
					"https://docs.google.com/gview?embedded=true&url=" + encodeURI(vm.obj.slides.url);
				var ref = new $cordovaInAppBrowser(url, '_blank', 'location=no,toolbar=yes,clearcache=no');
				ref.show();
			}
		}

		function getCertificate() {
			// if (isExpired()) {
			// 	return showExpirationAlert();
			// }
			var hours = Math.floor(vm.obj.duration / 60);
			var mins = (vm.obj.duration % 60);
			var duration = [];
			if (hours > 0) {
				duration.push("" + hours + " hora(s)");
			}
			if (mins > 0) {
				duration.push("" + mins + " minuto(s)");	
			}
			var strDuration = duration.join(" e ");

			var dateOptions = { weekday: 'long', year: 'numeric', month: 'long', day: 'numeric' };
			var substitutions = {
				"%userId%": vm.user.$id,
				"%talkId%": vm.obj.$id,
				"%name%": vm.user.name,
				"%talk%": vm.obj.title,
				"%location%": vm.obj.local,
				"%speaker%": vm.speaker.name,
				"%signature%": vm.speaker.signature ? vm.speaker.signature.url : "",
				"%address%": vm.obj.address,
				"%duration%": strDuration,
				"%date%": (new Date(vm.obj.startHour)).toLocaleString("pt-BR", dateOptions).replace("BRST", "")
			}
			var recipients = [
				{
					email: vm.user.email,
					name: vm.user.name
				}
			];
			var templateId = vm.obj.cert; //"fa7ccc4d-96d6-4562-9465-7e5bb431ee8f";
			$ionicLoading.show({
				template: 'Aguarde ...'
			});
			EmailService.send(recipients, templateId, substitutions).then(function() {
				$ionicLoading.hide();
				$ionicPopup.alert({
					title: 'Certificado obtido com sucesso.',
					template: 'O certificado foi gerado e enviado ao seu e-mail com sucesso.',
					okType: 'button-energized'
				});
			}, function (err) {
				console.log("ERROR", JSON.stringify(err));
				$ionicLoading.hide();
				$ionicPopup.alert({
					title: 'Desculpe',
					template: 'Não foi possível obter o certificado dessa palestra. Por favor, consulte a Profissionais S/A.',
					okType: 'button-energized'
				});
			});
		}

		function setLocal() {
			var localPopup = $ionicPopup.show({
				template: '<input type="text" ng-model="talk.local">',
				title: 'Definir Local',
				subTitle: 'Forneça o assento em que você está usando:',
				scope: $scope,
				buttons: [
					{ text: 'Cancelar' },
					{
						text: 'Enviar',
						type: 'button-energized',
						onTap: function (e) {
							if (!vm.local) {
								e.preventDefault();
							} else {
								return vm.local;
							}
						}
					}
				]
			});
			localPopup.then(function (res) {
				if (res) {
					if (typeof vm.obj.attendants[auth.uid] !== 'object') {
						vm.obj.attendants[auth.uid] = {
						};
					}
					vm.obj.attendants[auth.uid].local = res;
					vm.obj.$save().then(function () {
						$ionicPopup.alert({
							title: 'Obrigado',
							template: 'Localidade definida com sucesso.',
							okType: 'button-energized'
						});
					}).catch(function (err) {
						$ionicPopup.alert({
							title: 'Desculpe',
							template: 'Não foi possível atualizar sua localização',
							okType: 'button-energized'
						});
					});
				}
			});
		}

		function viewRatings() {
			$ionicPopup.show({
				templateUrl: 'modules/talks/talk.ratings.html',
				title: 'Avaliações',
				scope: $scope,
				buttons: [
					{
						text: 'Fechar',
						type: 'button-energized'
					}
				]
			}).then(function() {});
		}

		function rate() {
			// if (isExpired()) {
			// 	return showExpirationAlert();
			// }
			vm.rating = 0;
			vm.comments = '';
			var popup = $ionicPopup.show({
				templateUrl: 'modules/talks/talk.rating.html',
				title: 'Avaliação',
				subTitle: 'Avalie esta palestra',
				scope: $scope,
				buttons: [
					{ text: 'Cancelar' },
					{
						text: '<b>Avaliar</b>',
						type: 'button-energized',
						onTap: function (e) {
							if (!vm.rating) {
								e.preventDefault();
								return false;
							}
							return vm.rating;
						}
					}
				]
			});

			popup.then(function () {
				if (typeof vm.obj.attendants[auth.uid] !== 'object') {
					vm.obj.attendants[auth.uid] = {
					};
				} else {
					if (vm.obj.rating) {
						vm.obj.rating -= vm.obj.attendants[auth.uid].rating;
					}
					if (vm.obj.totalRating) {
						vm.obj.totalRating -= 1;
					}
				}
				if (!vm.obj.rating) {
					vm.obj.rating = 0;
				}
				if (!vm.obj.totalRating) {
					vm.obj.totalRating = 0;
				}
				vm.obj.rating += vm.rating;
				vm.obj.totalRating += 1;
				vm.obj.attendants[auth.uid].rating = vm.rating;
				vm.obj.attendants[auth.uid].comments = vm.comments;
				vm.obj.attendants[auth.uid].time = Date.now();
				vm.obj.$save();
			});
		}

		function getSpeaker() {
			vm.speaker = User(vm.obj.speaker);
		}

		function setLastTalk() {
			var obj = {
				title: vm.obj.title,
				description: vm.obj.description,
				$id: vm.obj.$id
			};
			console.log(vm.obj);
			TalkService.setLastTalk(obj);
		}

		function talkNotFound() {
			var popup = $ionicPopup.alert({
				title: 'Desculpe',
				template: 'Não foi possível encontrar esta palestra',
				okType: 'button-energized'
			});
			popup.then(function () {
				$state.go('app.menu.talks');
			});
		}

		function showExpirationAlert() {
			$ionicPopup.alert({
				title: 'Desculpe',
				template: 'A palestra já ocorreu. Esta opção não está mais disponível',
				okType: 'button-energized'
			});
		}

		function getTalk() {
			$ionicLoading.show({
				template: 'Carregando palestra...'
			});
			var id = $stateParams.id;
			console.log(!id || !id.length);
			if (!id || !id.length) {
				$ionicLoading.hide();
				return talkNotFound();
			}
			getAttendants();
			getLastQuestion();
			vm.obj = Talk(id);
			vm.obj.$loaded().then(function () {
				vm.time = null;
				if (vm.obj.startHour) {
					vm.time = new Date(Number(vm.obj.startHour));
				}
				if (!vm.obj || !vm.obj.$id) {
					vm.obj = null;
					throw new Error('Talk not found');
				} 
				getSpeaker();
				vm.amISpeaker = (vm.obj.speaker === auth.uid);
				setLastTalk();
			}).catch(function(err) {
				console.log("ERROR", err);
				return talkNotFound();
			}).finally(function () {
				$ionicLoading.hide();
			});
		}

		function isExpired() {
			return vm.obj && vm.obj.expireHour && Date.now() >= vm.obj.expireHour;
		}		

		function getAttendants() {
			var id = $stateParams.id;
			vm.attendants = Attendants(id, 4);
		}

		function getLastQuestion() {
			var talkId = $stateParams.id;
			vm.questions = Questions(talkId, 1);
		}

		function shareTalk() {
			// if (isExpired()) {
			// 	return showExpirationAlert();
			// }
			var message = "Estou assistindo a palestra \"" + vm.obj.title + "\" de " + vm.speaker.name + ": " + vm.obj.description + " #ProfissionaisSA";
			var subject = "Palestra \"" + vm.obj.title + "\"";
			var file = null;
			var link = "http://www.profissionaissa.com/app/palestras/" + vm.obj.$id;
			$cordovaSocialSharing
			.share(message, subject, file, link) // Share via native share sheet
			.then(function(result) {
				// Success!
			}, function(err) {
				// An error occured. Show a message to the user
			});
		}

		function activate() {
			getTalk();
		}
	}
})();
