(function() {
	'use strict';

	angular
		.module('psa.talks.controller', [])
		.controller('TalksController', TalksController);

	TalksController.$inject = ['User', 'Talks', '$ionicLoading', '$q', 'auth', '$scope', '$ionicBackdrop', '$timeout', '$state'];

	/* @ngInject */
	function TalksController(User, Talks, $ionicLoading, $q, auth, $scope, $ionicBackdrop, $timeout, $state) {
		var vm = this;

		vm.empty = empty;
		vm.selecting = {};
		vm.isSelecting = isSelecting;

		vm.locations = getCountries([]);

		$scope.$on('$ionicView.beforeEnter', activate);
		$scope.$on('$ionicView.enter', disallowSpeaker);

		$scope.$watch('talks.selecting.state', updateSelecting);
		$scope.$watch('talks.selecting.city', updateSelecting);

		$scope.$watch('talks.filter.state', function(val) {
		  if (val) {
		  	vm.filter.city = val.cities[0];
		  	console.log("FILTER", val);
		    vm.filtered = filterAvailable(vm.available, vm.filter);
		  }
		});

		$scope.$watch('talks.filter.city', function(val) {
		  if (val && val.id) {
		  	vm.filtered = filterAvailable(vm.available, vm.filter);
		  }
		});

		////////////////

		function isSelecting() {
		  var isIOS = ionic.Platform.isIOS();
			if (!isIOS) {
				return false;
			}
			return vm.selecting.city || vm.selecting.state;
		}

		function updateSelecting() {
		  var isIOS = ionic.Platform.isIOS();
			if (!isIOS) {
				return;
			}
			if (isSelecting()) {
  			$ionicBackdrop.retain();
			} else {
				$timeout(function() {
					if (!isSelecting()) {
						$ionicBackdrop.release();
					}
				}, 200);
			}
		}

		function getKey(type) {
			return type + 'Loaded';
		}

		function empty(type) {
			var key = getKey(type);
			if (!vm[key]) {
				return true;
			}
			return vm[type].length === 0;
		}

		function loaded(type) {
			var key = getKey(type);
			vm[key] = true;
		}

		function asSpeakerLoaded() {
			loaded('asSpeaker');
		}

		function asAttendantLoaded() {
			loaded('asAttendant');
		}

		function availableLoaded() {
			loaded('available');
		}

		function getTalksInfo() {
			vm.loading = true;
			$ionicLoading.show({
				template: 'Carregando palestras...'
			});
			vm.asSpeaker = Talks(auth.uid, 'asSpeaker');
			vm.asAttendant = Talks(auth.uid, 'asAttendant');
			vm.available = Talks(auth.uid, 'available');
			vm.asSpeaker.$loaded().then(asSpeakerLoaded);
			vm.asAttendant.$loaded().then(asAttendantLoaded);
			vm.available.$loaded().then(availableLoaded);

			return $q.all([vm.asSpeaker.$loaded(), vm.asAttendant.$loaded(), vm.available.$loaded()]).finally(function () {
				var taken = {};
				vm.asSpeaker.forEach(function(talk) {
					taken[talk.$id] = true;
				});
				vm.asAttendant.forEach(function(talk) {
					taken[talk.$id] = true;
				});
				vm.available = vm.available.filter(function(talk) {
					return !taken[talk.$id];
				});
				$ionicLoading.hide();
				vm.loading = false;
			});
		}

		function setVisibility() {
			vm.asAttendantShow = true;
			vm.asSpeakerShow = true;
			vm.availableShow = true;
		}

		function getCountries(available) {
			// build a key-based tree of objects with states and cities
			var map = available.reduce(function(states, talk, index) {
				var uf = talk.state;
				var city = talk.city;
				if (uf && city) {
					var state = states[uf] || {state: uf, uf: uf, cities: {
						"": {id: "-", city: "Todas"}
					}};
					states[uf] = state;
					state.cities[city] = state.cities[city] || {id: city, city: city};
				}
				return states;
			}, {});
			// compatibility (no Object.values present in safari)
			function objectValues(obj) {
				return Object.keys(obj).map(function(key) {
    			return obj[key];
				});
			}
			// remove keys (transforming objects into arrays) and sort arrays by name
			var states = objectValues(map).map(function(state) {
				state.cities = objectValues(state.cities);
				state.cities.sort(function(a, b) {
					if (a.id === "-" || b.id === "-") {
						return a.id.localeCompare(b.id);
					}
					return a.city.localeCompare(b.city);
				});
				return state;
			});
			states.sort(function(a, b) {
				return a.state.localeCompare(b.state);
			});
			states.unshift({state: "Todos", uf: "", cities: [{id: "-", city: "Todas"}]});

			// return the expected object format
			return {
				"countries": [
					{
						"country": "Brasil",
						"states": states
					}
				]
			};
		}

		function filterAvailable(available, filter) {
			var now = Date.now();
			return available.filter(function(talk) {
				if (talk.expireHour && now >= talk.expireHour) {
					return false;
				}
				if (filter.state && filter.state.uf && talk.state !== filter.state.uf) {
					return false;
				}
				if (filter.city && filter.city.id !== "-" && talk.city !== filter.city.city) {
					return false;
				}
				return true;
			});
		}

		function disallowSpeaker() {
			function checkSpeakerMode() {
				return JSON.parse(localStorage.getItem("speakerMode"));
			}
			if (checkSpeakerMode()) {
				$state.go("app.menu.lecturing");
				return;
			}
		}

		function activate() {
			setVisibility();
			getTalksInfo().then(function() {
				vm.locations = getCountries(vm.available);
				vm.filter = {
					country: vm.locations.countries[0],
					state: vm.locations.countries[0].states[0],
					city: vm.locations.countries[0].states[0].cities[0]
				};
				vm.filtered = filterAvailable(vm.available, vm.filter);
			})
		}
	}
})();
