(function() {
	'use strict';

	angular
		.module('psa.user.controller', [])
		.controller('UserController', UserController);

	UserController.$inject = ['User', 'auth', '$ionicPlatform', 'Talks', '$stateParams', '$ionicLoading', '$ionicPopup', '$ionicHistory', '$cordovaEmailComposer'];

	/* @ngInject */
	function UserController(User, auth, $ionicPlatform, Talks, $stateParams, $ionicLoading, $ionicPopup, $ionicHistory, $cordovaEmailComposer) {
		var vm = this;

		vm.addToContacts = addToContacts;
		vm.sendEmail = sendEmail;
		vm.talk = null;

		activate();

		////////////////

		function contactErr(err) {
			console.error(err);
			$ionicPopup.alert({
				title: 'Desculpe',
				template: 'Não foi possível salvar o contato. Por favor, tente novamente',
				okType: 'button-energized'
			});
		}

		function contactSuccess() {
			$ionicPopup.alert({
				title: 'Contato salvo',
				template: 'O contato foi salvo com sucesso em sua lista.',
				okType: 'button-energized'
			});
		}

		function addToContacts() {
			$ionicPlatform.ready(function () {
				if (angular.isDefined(Contact)) {
					var contact = new Contact();
					contact.displayName = vm.profile.name;
					contact.nickname = vm.profile.name;

					// if (vm.profile.phone) {
					// 	contact.phoneNumbers = [
					// 		{ type: 'work', value: vm.profile.phone, pref: true }
					// 	];
					// }
					// if (vm.profile.phone) {
					// 	contact.phoneNumbers = [
					// 		{ type: 'mobile', value: vm.profile.celphone, pref: true }
					// 	];
					// }

					if (vm.profile.picture) {
						contact.photos = [
							{ type: 'base64', value: vm.profile.picture, pref: true }
						];
					}
					if (vm.profile.email) {
						contact.emails = [
							{ type: 'email', value: vm.profile.email, pref: true }
						];
					}
					contact.save(contactSuccess, contactErr);
				} else {
					contactErr();
				}
			});
		}

		function userNotFound() {
			$ionicLoading.hide();
			popup = $ionicPopup.alert({
				title: 'Usuário não encontrado',
				template: 'Não foi possível encontrar este usuário',
				okType: 'button-energized'
			});
			popup.then(function () {
				$ionicHistory.goBack();
			});

		}

		function getTalks(id) {
			var talkId = $stateParams.talk;
			vm.talksAsSpeaker = new Talks(id, 'asSpeaker');
			vm.talksAsSpeaker.$loaded().then(function () {
				vm.talksAsSpeaker.forEach(function(_talk) {
					if (_talk.$id === talkId) {
						vm.talk = _talk;
					}
				});
			});
			vm.talksAsAttendant = new Talks(id, 'asAttendant');
			vm.talksAsAttendant.$loaded().then(function () {
				vm.talksAsAttendant.forEach(function(_talk) {
					if (_talk.$id === talkId) {
						vm.talk = _talk;
					}
				});
			});
		}

		function getProfile() {
			$ionicLoading.show({
				template: 'Buscando perfil...'
			});
			var id = $stateParams.id;
			if (!id || !id.length) {
				return userNotFound();
			}
			vm.profile = new User(id);
			getTalks(id);
			vm.profile.$loaded().then(function () {
				console.log(auth, vm.profile);
				if (auth.uid === vm.profile.$id) {
					vm.amI = true;
				}
			}).catch(function () {
				userNotFound();
			}).finally(function () {
				$ionicLoading.hide();
			});
		}

		function sendEmail(to) {
		  var email = {
		    to: to,
		    subject: 'Contato pelo aplicativo Profissionais S/A',
		    body: 'Escreva aqui sua mensagem para este profissional',
		    isHtml: false
		  };

			$cordovaEmailComposer.isAvailable().then(function() {
			  return $cordovaEmailComposer.open(email);
			}, function () {
			  // not available
			});
		}


		function activate() {
			getProfile();
		}
	}
})();
