(function() {
	'use strict';

	angular
		.module('psa.user.edit.controller', [])
		.controller('UserEditController', UserEditController);

	UserEditController.$inject = ['Auth', 'UploadImage', '$ionicPopup', '$log', '$cordovaCamera', '$ionicActionSheet', 'auth', 'User', '$ionicLoading'];

	/* @ngInject */
	function UserEditController(Auth, UploadImage, $ionicPopup, $log, $cordovaCamera, $ionicActionSheet, auth, User, $ionicLoading) {
		var vm = this;

		vm.updatePassword = updatePassword;
		vm.selectPicture = selectPicture;
		vm.update = update;

		activate();

		////////////////


		function showErrors(errors) {
			if (!errors.length) {
				return true;
			}
			var template = '';
			for (var e in errors) {
				var error = errors[e];
				template += error + '<br>';
			}
			$ionicPopup.alert({
				title: 'Desculpe',
				subTitle: 'Houve um ou mais erros em seu cadastro',
				template: template,
				okType: 'button-energized'

			});
			return false;
		}

		function validate() {
			var errors = [];
			if (!vm.user.pictureUrl) {
				errors.push('<strong>Foto</strong> é obrigatório');
			}
			if (!vm.user.name) {
				errors.push('<strong>Nome</strong> é obrigatório');
			}
			if (!vm.user.email) {
				errors.push('<strong>E-mail</strong> é obrigatório');				
			}
			if (!vm.user.phone) {
				errors.push('<strong>Telefone</strong> é obrigatório');				
			}
			return showErrors(errors);
		}

		function validatePassword() {
			var errors = [];
			if (!vm.password || vm.password.length < 6) {
				errors.push('<strong>Senha</strong> deve conter no mínimo 6 caracteres');
			}
			if (vm.password !== vm.confirmPassword) {
				errors.push('<strong>Senha</strong> e <strong>Confirmação de Senha</strong> não conferem.');
			}
			return showErrors(errors);
		}

		function updatePassword() {
			if (validatePassword()) {
				Auth.$signInWithEmailAndPassword(auth.email, vm.currentPassword).then(function () {
					Auth.$updatePassword(vm.password).then(function () {
						$ionicPopup.alert({
							title: 'Senha atualizada',
							template: 'Sua senha foi atualizada com sucesso.',
							okType: 'button-energized'
						});
						delete vm.password;
						delete vm.currentPassword;
						delete vm.confirmPassword;
					}).catch(function (err) {
						$ionicPopup.alert({
							title: 'Desculpe',
							template: 'Não foi possível atualizar a sua senha.',
							okType: 'button-energized'
						});
					});
				}).catch(function () {
					$ionicPopup.alert({
						title: 'Desculpe',
						template: 'Sua senha antiga não confere.',
						okType: 'button-energized'
					});
				});
			}
		}

		function getPicture(type) {
			$cordovaCamera.getPicture({
				destinationType: $cordovaCamera.DestinationType.FILE_URI,
				encodingType: $cordovaCamera.EncodingType.JPEG,
				sourceType: type,
				allowEdit: true,
				targetWidth: 250,
				targetHeight: 250
			}).then(function (res) {
				updatePicture(res);
			}).catch(function (error) {
				$log.error(error);
			});
		}

		function updatePicture(pic) {
			vm.progress = '0%';
			$ionicLoading.show({
				template: 'Atualizando Imagem... ' + vm.progress 
			});
			console.log("PIC", pic);
			UploadImage.uploadImage(pic, 'pictures', auth.uid, function (progress) {
				vm.progress = progress + '%';
			}).then(function (pictureUrl) {
				vm.original.pictureUrl = pictureUrl;
				return vm.original.$save();
			}).then(function() {
				console.log("ORIG PIC URL", vm.original.pictureUrl);
				$ionicPopup.alert({
					title: 'Foto Atualizada',
					template: 'Sua foto foi atualizada com sucesso.',
					okType: 'button-energized'
				});
			}).catch(function (err) {
				$ionicPopup.alert({
					title: 'Desculpe',
					template: 'Houve um erro ao tentar atualizar a sua foto',
					okType: 'button-energized'
				});
			}).finally(function () {
				$ionicLoading.hide();
			});
		}

		function selectPicture() {
			$ionicActionSheet.show({
				buttons: [
					{ text: '<i class="icon ion-camera dark"></i> Selecionar da câmera' },
					{ text: '<i class="icon ion-image energized"></i> Selecionar do álbum' }
				],
				titleText: 'Sua foto',
				cancelText: 'Cancelar',
				buttonClicked: function (index) {
					var type;
					switch (index) {
						case 0:
							type = $cordovaCamera.PictureSourceType.CAMERA;
							break;
						case 1:
							type = $cordovaCamera.PictureSourceType.PHOTOLIBRARY;
							break;
					}
					getPicture(type);
					return true;
				}
			});
		}

		function getUserInfo() {
			$ionicLoading.show({
				template: 'Buscando perfil...'
			});
			vm.original = User(auth.uid);
			vm.original.$loaded().finally(function () {
				vm.user = {};
				angular.forEach(vm.original, function(value, key) {
					vm.user[key] = value;
				});
				// ignore picture on the form entry
				delete vm.user.pictureUrl;

				$ionicLoading.hide();
			});
			vm.auth = auth;
		}

		function update() {
			angular.forEach(vm.user, function(value, key) {
				vm.original[key] = value;
			});
			vm.original.$save().then(function () {
				$ionicPopup.alert({
					title: 'Dados atualizados',
					template: 'Os dados foram atualizados com sucesso.',
					okType: 'button-energized'
				});
			});
		}

		function activate() {
			getUserInfo();
		}
	}
})();
